@@ -1,8 +1,10 @@
-# Copyright 2000-2005 The Apache Software Foundation
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+# please insert nothing before this line: -*- mode: cperl; cperl-indent-level: 4; cperl-continued-statement-offset: 4; indent-tabs-mode: nil -*-
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
 #
 #     http://www.apache.org/licenses/LICENSE-2.0
 #
@@ -26,6 +28,42 @@ use File::Basename;
 use ExtUtils::Embed ();
 use File::Copy ();
 
+BEGIN {				# check for a sane ExtUtils::Embed
+    unless ($ENV{MP_USE_MY_EXTUTILS_EMBED}) {
+	my ($version, $path)=(ExtUtils::Embed->VERSION,
+			      $INC{q{ExtUtils/Embed.pm}});
+	my $msg=<<"EOF";
+I have found ExtUtils::Embed $version at
+
+  $path
+
+This is probably not the right one for this perl version. Please make sure
+there is only one version of this module installed and that it is the one
+that comes with this perl version.
+
+If you insist on using the ExtUtils::Embed as is set the environment
+variable MP_USE_MY_EXTUTILS_EMBED=1 and try again.
+
+EOF
+	if (eval {require Module::CoreList}) {
+	    my $req=$Module::CoreList::version{$]}->{q/ExtUtils::Embed/};
+	    die "Please repair your Module::CoreList" unless $req;
+	    unless ($version eq $req) {
+		$msg.=("Details: expecting ExtUtils::Embed $req ".
+		       "(according to Module::CoreList)\n\n");
+		die $msg;
+	    }
+	}
+	else {
+	    my $req=$Config{privlib}.'/ExtUtils/Embed.pm';
+	    unless ($path eq $req) {
+		$msg.="Details: expecting ExtUtils::Embed at $req\n\n";
+		die $msg;
+	    }
+	}
+    }
+}
+
 use constant IS_MOD_PERL_BUILD => grep 
     { -e "$_/Makefile.PL" && -e "$_/lib/mod_perl2.pm" } qw(. ..);
 
@@ -38,6 +76,7 @@ use constant OPENBSD => $^O eq 'openbsd';
 use constant WIN32   => $^O eq 'MSWin32';
 
 use constant MSVC => WIN32() && ($Config{cc} eq 'cl');
+use constant DMAKE => WIN32() && ($Config{make} eq 'dmake');
 
 use constant REQUIRE_ITHREADS => grep { $^O eq $_ } qw(MSWin32);
 use constant PERL_HAS_ITHREADS =>
@@ -104,13 +143,15 @@ sub httpd_is_source_tree {
         defined $prefix && -d $prefix && -e "$prefix/CHANGES";
 }
 
-# try to find the apxs utility, set $apxs to the path if found,
+# try to find the apxs utility, set $self->{MP_APXS} to the path if found,
 # otherwise to ''
-my $apxs; # undef so we know we haven't tried to set it yet
 sub find_apxs_util {
     my $self = shift;
 
-    $apxs = ''; # not found
+    if (not defined $self->{MP_APXS}) {
+        $self->{MP_APXS} = ''; # not found
+    }
+
     my @trys = ($Apache2::Build::APXS,
                 $self->{MP_APXS},
                 $ENV{MP_APXS});
@@ -140,7 +181,7 @@ sub find_apxs_util {
         next unless ($apxs_try = $_);
         chomp $apxs_try;
         if (-x $apxs_try) {
-            $apxs = $apxs_try;
+            $self->{MP_APXS} = $apxs_try;
             last;
         }
     }
@@ -170,7 +211,7 @@ sub ap_destdir {
 sub apxs {
     my $self = shift;
 
-    $self->find_apxs_util() unless defined $apxs;
+    $self->find_apxs_util() unless defined $self->{MP_APXS};
 
     my $is_query = (@_ == 2) && ($_[0] eq '-q');
 
@@ -184,7 +225,7 @@ sub apxs {
         }
     }
 
-    unless ($apxs) {
+    unless ($self->{MP_APXS}) {
         my $prefix = $self->{MP_AP_PREFIX} || "";
         return '' unless -d $prefix and $is_query;
         my $val = $apxs_query{$_[1]};
@@ -192,15 +233,15 @@ sub apxs {
     }
 
     my $devnull = devnull();
-    my $val = qx($apxs @_ 2>$devnull);
+    my $val = qx($self->{MP_APXS} @_ 2>$devnull);
     chomp $val if defined $val;
 
     unless ($val) {
         # do we have an error or is it just an empty value?
-        my $error = qx($apxs @_ 2>&1);
+        my $error = qx($self->{MP_APXS} @_ 2>&1);
         chomp $error if defined $error;
         if ($error) {
-            error "'$apxs @_' failed:";
+            error "'$self->{MP_APXS} @_' failed:";
             error $error;
         }
         else {
@@ -227,7 +268,8 @@ sub apxs_extra_cflags {
 
 sub apxs_extra_cppflags {
     my $who = caller_package(shift);
-    my $flags = $who->apxs('-q' => 'EXTRA_CPPFLAGS');
+    my $flags = $who->apxs('-q' => 'EXTRA_CPPFLAGS') ." ".
+        $who->apxs('-q' => 'NOTEST_CPPFLAGS');
     $flags =~ s/\"/\\\"/g;
     $flags;
 }
@@ -238,11 +280,12 @@ sub caller_package {
 }
 
 my %threaded_mpms = map { $_ => 1 }
-        qw(worker winnt beos mpmt_os2 netware leader perchild threadpool);
+        qw(worker winnt beos mpmt_os2 netware leader perchild threadpool
+           dynamic);
 sub mpm_is_threaded {
     my $self = shift;
     my $mpm_name = $self->mpm_name();
-    return $threaded_mpms{$mpm_name} || 0;
+    return exists $threaded_mpms{$mpm_name} ? 1 : 0;
 }
 
 sub mpm_name {
@@ -250,10 +293,23 @@ sub mpm_name {
 
     return $self->{mpm_name} if $self->{mpm_name};
 
+    if ($self->httpd_version =~ /^(\d+)\.(\d+)\.(\d+)/) {
+	delete $threaded_mpms{dynamic} if $self->mp_nonthreaded_ok;
+	return $self->{mpm_name} = 'dynamic' if ($1*1000+$2)*1000+$3>=2003000;
+    }
+
     # XXX: hopefully apxs will work on win32 one day
     return $self->{mpm_name} = 'winnt' if WIN32;
 
-    my $mpm_name = $self->apxs('-q' => 'MPM_NAME');
+    my $mpm_name;
+
+    # httpd >= 2.3
+    if ($self->httpd_version_as_int =~ m/^2[3-9]\d+/) {
+        $mpm_name = 'dynamic';
+    }
+    else {
+        $mpm_name = $self->apxs('-q' => 'MPM_NAME');
+    }
 
     # building against the httpd source dir
     unless (($mpm_name and $self->httpd_is_source_tree)) {
@@ -524,11 +580,11 @@ sub ap_ccopts {
         $self->{MP_DEBUG} = 1;
         if ($self->perl_config('gccversion')) {
             #same as --with-maintainter-mode
-            $ccopts .= " $Wall -DAP_DEBUG";
-            $ccopts .= " -DAP_HAVE_DESIGNATED_INITIALIZER";
+            $ccopts .= " $Wall";
         }
 
-        if ($self->has_gcc_version('3.3.2') && 
+        if (!OPENBSD &&
+            $self->has_gcc_version('3.3.2') && 
             $ccopts !~ /declaration-after-statement/) {
             debug "Adding -Wdeclaration-after-statement to ccopts";
             $ccopts .= " -Wdeclaration-after-statement";
@@ -555,6 +611,14 @@ sub ap_ccopts {
         $ccopts .= " -DMP_TRACE";
     }
 
+    if ($self->has_gcc_version('5.0.0') && $ccopts !~ /-fgnu89-inline/) {
+        $ccopts .= " -fgnu89-inline";
+    }
+
+    if ($self->has_clang && $ccopts !~ /-std=gnu89/) {
+        $ccopts .= " -std=gnu89";
+    }
+
     # make sure apr.h can be safely included
     # for example Perl's included -D_GNU_SOURCE implies
     # -D_LARGEFILE64_SOURCE on linux, but this won't happen on
@@ -562,6 +626,9 @@ sub ap_ccopts {
     my $extra_cppflags = $self->apxs_extra_cppflags;
     $ccopts .= " " . $extra_cppflags;
 
+    # Make sure the evil AP_DEBUG is not defined when building mod_perl
+    $ccopts =~ s/ ?-DAP_DEBUG\b//;
+
     $ccopts;
 }
 
@@ -582,6 +649,16 @@ sub has_gcc_version {
     return cmp_tuples(\@tuples, \@r_tuples) == 1;
 }
 
+sub has_clang {
+    my $self = shift;
+
+    my $has_version = $self->perl_config('gccversion');
+
+    return 0 unless $has_version;
+
+    return $has_version =~ m/Clang/;
+}
+
 sub cmp_tuples {
     my ($num_a, $num_b) = @_;
 
@@ -864,6 +941,7 @@ sub new {
     my $self = bless {
         cwd        => Cwd::fastcwd(),
         MP_LIBNAME => 'mod_perl',
+        MP_APXS    => undef, # so we know we haven't tried to set it yet
         @_,
     }, $class;
 
@@ -884,7 +962,7 @@ my %default_files = (
 
 sub clean_files {
     my $self = shift;
-    [map { $self->default_file($_) } keys %default_files];
+    [sort map { $self->default_file($_) } keys %default_files];
 }
 
 sub default_file {
@@ -895,6 +973,10 @@ sub default_file {
 
 sub file_path {
     my $self = shift;
+
+    # work around when Apache2::BuildConfig has not been created yet
+    return unless $self && $self->{cwd};
+
     my @files = map { m:^/: ? $_ : join('/', $self->{cwd}, $_) } @_;
     return wantarray ? @files : $files[0];
 }
@@ -1116,7 +1198,9 @@ sub apru_link_flags {
     # first use apu_config_path and then apr_config_path in order to
     # resolve the symbols right during linking
     for ($self->apu_config_path, $self->apr_config_path) {
-        if (my $link = $_ && -x $_ && qx{$_ --link-ld --libs}) {
+        my $flags = '--link-ld --libs';
+        $flags .= ' --ldflags' unless (WIN32);
+        if (my $link = $_ && -x $_ && qx{$_ $flags}) {
             chomp $link;
 
             # Change '/path/to/libanything.la' to '-L/path/to -lanything'
@@ -1376,8 +1460,17 @@ sub httpd_version {
             my $major = $1;
             my $minor = (split /\s+/, scalar(<$fh>))[-1];
             my $patch = (split /\s+/, scalar(<$fh>))[-1];
-            my $string = (split /\s+/, scalar(<$fh>))[-1];
-            $version = join '.', $major, $minor, "$patch$string";
+
+            my ($define, $macro, $dev) = (split /\s+/, scalar(<$fh>));
+            
+            if ($macro =~ /AP_SERVER_DEVBUILD_BOOLEAN/ && $dev eq '1') {
+                $dev = "-dev";
+            }
+            else {
+                $dev = "";   
+            }
+
+            $version = join '.', $major, $minor, "$patch$dev";
             $version =~ s/\"//g;
             last;
         }
@@ -1455,7 +1548,8 @@ sub make_xs {
     my @files;
     my @xs_targ;
 
-    while (my ($name, $xs) = each %{ $self->{XS} }) {
+    foreach my $name (sort keys %{ $self->{XS} }) {
+        my $xs = $self->{XS}->{$name};
         #Foo/Bar.xs => Foo_Bar.c
         (my $c = $xs) =~ s:.*?WrapXS/::;
         $c =~ s:/:_:g;
@@ -1499,7 +1593,7 @@ my $mm_replace = join '|', keys %perl_config_pm_alias;
 my %perl_config_pm_alias_values = reverse %perl_config_pm_alias;
 my @perl_config_pm_alias_values = keys %perl_config_pm_alias_values;
 
-my @perl_config_pm = (@perl_config_pm_alias_values, qw(cc cpprun
+my @perl_config_pm = sort(@perl_config_pm_alias_values, qw(cc cpprun
     rm ranlib lib_ext obj_ext cccdlflags lddlflags optimize));
 
 sub mm_replace {
@@ -1519,8 +1613,11 @@ sub make_tools {
 
     require ExtUtils::MakeMaker;
     my $mm = bless { @mm_init_vars }, 'MM';
-    $mm->init_main;
-    $mm->init_others;
+
+    # Fake initialize MakeMaker
+    foreach my $m (qw(init_main init_others init_tools)) {
+        $mm->$m() if $mm->can($m);
+    }
 
     for (qw(rm_f mv ld ar cp test_f)) {
         my $val = $mm->{"\U$_"};
@@ -1579,7 +1676,10 @@ sub dynamic_link_MSWin32 {
     return $self->dynamic_link_header_default .
         "\t$defs" .
         ($symbols ? ' \\' . "\n\t-pdb:$symbols" : '') .
-        ' -out:$@' . "\n\n";
+        ' -out:$@' . "\n\t" .
+        'if exist $(MODPERL_MANIFEST_LOCATION)' . " \\\n\t" .
+        'mt /nologo /manifest $(MODPERL_MANIFEST_LOCATION)' . " \\\n\t" .
+        '/outputresource:$@;2' . "\n\n";
 }
 
 sub dynamic_link_aix {
@@ -1644,7 +1744,11 @@ sub apache_corelib_cygwin {
 sub apache_libs_MSWin32 {
     my $self = shift;
     my $prefix = $self->apxs(-q => 'PREFIX') || $self->dir;
-    my @libs = map { "$prefix/lib/lib$_.lib" } qw(apr aprutil httpd);
+    my $lib = catdir $prefix, 'lib';
+    opendir(my $dir, $lib) or die qq{Cannot opendir $lib: $!};
+    my @libs = map {catfile($lib, $_)}
+        grep /^lib(apr|aprutil|httpd)\b\S*?\.lib$/, readdir $dir;
+    closedir $dir;
     "@libs";
 }
 
@@ -1804,7 +1908,8 @@ EOI
     #XXX short-term compat for Apache::TestConfigPerl
     $libs{shared} = $libs{dso};
 
-    while (my ($type, $lib) = each %libs) {
+    foreach my $type (sort keys %libs) {
+        my $lib = $libs{$type};
         print $fh $self->canon_make_attr("lib_$type", $libs{$type});
     }
 
@@ -1821,6 +1926,12 @@ EOI
     if ($self->is_dynamic && (my $libs = $self->modperl_libpath)) {
         print $fh $self->canon_make_attr('lib_location', $libs);
 
+        # Visual Studio 8 on Win32 uses manifest files
+        if (WIN32) {
+            (my $manifest = $libs) =~ s/\.lib$/.so.manifest/;
+            print $fh $self->canon_make_attr('manifest_location', $manifest);
+        }
+
         print $fh $self->canon_make_attr('ap_libdir',
             $self->ap_destdir(catdir $self->{MP_AP_PREFIX}, 'lib')
         );
@@ -1852,7 +1963,7 @@ EOI
     }
 
     my @libs;
-    for my $type (map { uc } keys %libs) {
+    for my $type (sort map { uc } keys %libs) {
         next unless $self->{"MP_USE_$type"};
         # on win32 mod_perl.lib must come after mod_perl.so
         $type eq 'STATIC'
@@ -1889,6 +2000,11 @@ EOF
 
     print $fh $install;
 
+    print $fh <<'EOF' if DMAKE;
+
+.USESHELL :
+EOF
+
     print $fh <<'EOF';
 
 .SUFFIXES: .xs .c $(MODPERL_OBJ_EXT) .lo .i .s
@@ -1921,7 +2037,7 @@ EOF
 clean:
 	$(MODPERL_RM_F) *.a *.so *.xsc \
 	$(MODPERL_LIBNAME).exp $(MODPERL_LIBNAME).lib \
-	*$(MODPERL_OBJ_EXT) *.lo *.i *.s *.pdb \
+	*$(MODPERL_OBJ_EXT) *.lo *.i *.s *.pdb *.manifest \
 	$(MODPERL_CLEAN_FILES) \
 	$(MODPERL_XS_CLEAN_FILES)
 
@@ -2042,6 +2158,7 @@ sub includes {
 }
 
 sub inc {
+    local $_;
     my @includes = map { "-I$_" } @{ shift->includes };
     "@includes";
 }
@@ -2116,7 +2233,8 @@ sub has_large_files_conflict {
     # with it is that we didn't have such a case yet, but may need to
     # deal with it later
 
-    return $perl_lfs64 ^ $apr_lfs64;
+    return 0;
+    # $perl_lfs64 ^ $apr_lfs64;
 }
 
 # if perl is built with uselargefiles, but apr not, the build won't
