@@ -64,7 +64,7 @@ modperl_mgv_t *modperl_mgv_new(apr_pool_t *p)
 
 #define modperl_mgv_hash(mgv)                   \
     PERL_HASH(mgv->hash, mgv->name, mgv->len)
- /* MP_TRACE_h(MP_FUNC, "%s...hash=%ld\n", mgv->name, mgv->hash) */
+ /* MP_TRACE_h(MP_FUNC, "%s...hash=%ld", mgv->name, mgv->hash) */
 
 modperl_mgv_t *modperl_mgv_compile(pTHX_ apr_pool_t *p,
                                    register const char *name)
@@ -138,11 +138,11 @@ GV *modperl_mgv_lookup(pTHX_ modperl_mgv_t *symbol)
             }
         }
         else {
-            return Nullgv;
+            return (GV *)NULL;
         }
     }
 
-    return Nullgv;
+    return (GV *)NULL;
 }
 
 #ifdef USE_ITHREADS
@@ -156,7 +156,7 @@ MP_INLINE GV *modperl_mgv_lookup_autoload(pTHX_ modperl_mgv_t *symbol,
         return gv;
     }
 
-    /* 
+    /*
      * this VirtualHost has its own parent interpreter
      * must require the module again with this server's THX
      */
@@ -184,7 +184,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
 {
     CV *cv;
     GV *gv;
-    HV *stash = Nullhv;
+    HV *stash = (HV *)NULL;
     char *handler_name = "handler";
     char *tmp;
 
@@ -211,14 +211,12 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
         handler->name    = NULL;
         handler->mgv_obj = modperl_handler_anon_next(aTHX_ p);
         modperl_handler_anon_add(aTHX_ handler->mgv_obj, cv);
-        MP_TRACE_h(MP_FUNC, "[%s] new anon handler",
-                   modperl_pid_tid(p));
+        MP_TRACE_h(MP_FUNC, "new anon handler");
 #else
         SvREFCNT_inc(cv);
         handler->cv      = cv;
         handler->name    = NULL;
-        MP_TRACE_h(MP_FUNC, "[%s] new cached-cv anon handler",
-                   modperl_pid_tid(p));
+        MP_TRACE_h(MP_FUNC, "new cached-cv anon handler");
 #endif
 
         FREETMPS;LEAVE;
@@ -241,37 +239,37 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
 
             handler->mgv_obj = modperl_mgv_compile(aTHX_ p, package + 1);
             gv = modperl_mgv_lookup(aTHX_ handler->mgv_obj);
-            obj = gv ? GvSV(gv) : Nullsv;
+            obj = gv ? GvSV(gv) : (SV *)NULL;
 
             if (SvTRUE(obj)) {
                 if (SvROK(obj) && sv_isobject(obj)) {
                     stash = SvSTASH(SvRV(obj));
                     MpHandlerOBJECT_On(handler);
-                    MP_TRACE_h(MP_FUNC, "handler object %s isa %s\n",
+                    MP_TRACE_h(MP_FUNC, "handler object %s isa %s",
                                package, HvNAME(stash));
                 }
                 else {
-                    MP_TRACE_h(MP_FUNC, "%s is not an object, pv=%s\n",
+                    MP_TRACE_h(MP_FUNC, "%s is not an object, pv=%s",
                                package, SvPV_nolen(obj));
                     return 0;
                 }
             }
             else {
-                MP_TRACE_h(MP_FUNC, "failed to thaw %s\n", package);
+                MP_TRACE_h(MP_FUNC, "failed to thaw %s", package);
                 return 0;
             }
         }
 
         if (!stash) {
             if ((stash = gv_stashpvn(package, package_len, FALSE))) {
-                MP_TRACE_h(MP_FUNC, "handler method %s isa %s\n",
+                MP_TRACE_h(MP_FUNC, "handler method %s isa %s",
                            name, HvNAME(stash));
             }
         }
     }
     else {
         if ((cv = get_cv(name, FALSE))) {
-            handler->attrs = (U32)MP_CODE_ATTRS(cv);
+            handler->attrs = *modperl_code_attrs(aTHX_ cv);
             handler->mgv_cv =
                 modperl_mgv_compile(aTHX_ p, HvNAME(GvSTASH(CvGV(cv))));
             modperl_mgv_append(aTHX_ p, handler->mgv_cv, GvNAME(CvGV(cv)));
@@ -284,11 +282,11 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
     if (!stash && MpHandlerAUTOLOAD(handler)) {
         if (!modperl_perl_module_loaded(aTHX_ name)) { /* not in %INC */
             MP_TRACE_h(MP_FUNC,
-                       "package %s not in %INC, attempting to load it\n",
+                       "package %s not in %INC, attempting to load it",
                        name);
 
             if (modperl_require_module(aTHX_ name, logfailure)) {
-                MP_TRACE_h(MP_FUNC, "loaded %s package\n", name);
+                MP_TRACE_h(MP_FUNC, "loaded %s package", name);
             }
             else {
                 if (logfailure) {
@@ -297,13 +295,13 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
                 }
                 else {
                     /* the caller handles the error checking */
-                    MP_TRACE_h(MP_FUNC, "failed to load %s package\n", name);
+                    MP_TRACE_h(MP_FUNC, "failed to load %s package", name);
                     return 0;
                 }
             }
         }
         else {
-            MP_TRACE_h(MP_FUNC, "package %s seems to be loaded\n", name);
+            MP_TRACE_h(MP_FUNC, "package %s seems to be loaded", name);
         }
     }
 
@@ -312,7 +310,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
      * module was loaded, preventing from loading the module
      */
     if (!(stash || (stash = gv_stashpv(name, FALSE)))) {
-        MP_TRACE_h(MP_FUNC, "%s's stash is not found\n", name);
+        MP_TRACE_h(MP_FUNC, "%s's stash is not found", name);
         return 0;
     }
 
@@ -324,8 +322,8 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
         if (!stash) {
             return 0;
         }
-        
-        
+
+
         if (MpHandlerMETHOD(handler) && !handler->mgv_obj) {
             char *name = HvNAME(stash);
             if (!name) {
@@ -334,14 +332,13 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
             modperl_mgv_new_name(handler->mgv_obj, p, name);
         }
 
-        handler->attrs = (U32)MP_CODE_ATTRS(cv);
+        handler->attrs = *modperl_code_attrs(aTHX_ cv);
         /* note: this is the real function after @ISA lookup */
         handler->mgv_cv = modperl_mgv_compile(aTHX_ p, HvNAME(GvSTASH(gv)));
         modperl_mgv_append(aTHX_ p, handler->mgv_cv, handler_name);
 
         MpHandlerPARSED_On(handler);
-        MP_TRACE_h(MP_FUNC, "[%s] found `%s' in class `%s' as a %s\n",
-                   modperl_pid_tid(p),
+        MP_TRACE_h(MP_FUNC, "found `%s' in class `%s' as a %s",
                    handler_name, HvNAME(stash),
                    MpHandlerMETHOD(handler) ? "method" : "function");
         MODPERL_MGV_DEEP_RESOLVE(handler, p);
@@ -359,7 +356,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
 #ifdef MP_TRACE
     /* complain only if the class was actually loaded/created */
     if (stash) {
-        MP_TRACE_h(MP_FUNC, "`%s' not found in class `%s'\n",
+        MP_TRACE_h(MP_FUNC, "`%s' not found in class `%s'",
                    handler_name, name);
     }
 #endif
@@ -418,7 +415,7 @@ int modperl_mgv_require_module(pTHX_ modperl_mgv_t *symbol,
         modperl_mgv_as_string(aTHX_ symbol, p, 1);
 
     if (modperl_require_module(aTHX_ package, TRUE)) {
-        MP_TRACE_h(MP_FUNC, "reloaded %s for server %s\n",
+        MP_TRACE_h(MP_FUNC, "reloaded %s for server %s",
                    package, modperl_server_desc(s, p));
         return TRUE;
     }
@@ -452,7 +449,7 @@ static void modperl_hash_handlers(pTHX_ apr_pool_t *p, server_rec *s,
 #ifdef USE_ITHREADS
             if ((MpSrvPARENT(scfg) && MpSrvAUTOLOAD(scfg))
                 && !modperl_mgv_lookup(aTHX_ handler->mgv_cv)) {
-                /* 
+                /*
                  * this VirtualHost has its own parent interpreter
                  * must require the module again with this server's THX
                  */
@@ -460,7 +457,7 @@ static void modperl_hash_handlers(pTHX_ apr_pool_t *p, server_rec *s,
                                            s, p);
             }
 #endif
-            MP_TRACE_h(MP_FUNC, "%s already resolved in server %s\n",
+            MP_TRACE_h(MP_FUNC, "%s already resolved in server %s",
                        modperl_handler_name(handler),
                        modperl_server_desc(s, p));
         }
@@ -479,10 +476,7 @@ static int modperl_hash_handlers_dir(apr_pool_t *p, server_rec *s,
 {
     int i;
     modperl_config_dir_t *dir_cfg = (modperl_config_dir_t *)cfg;
-#ifdef USE_ITHREADS
-    MP_dSCFG(s);
-    MP_dSCFG_dTHX;
-#endif
+    dTHXa(data);
 
     if (!dir_cfg) {
         return 1;
@@ -500,7 +494,7 @@ static int modperl_hash_handlers_srv(apr_pool_t *p, server_rec *s,
 {
     int i;
     modperl_config_srv_t *scfg = (modperl_config_srv_t *)cfg;
-    MP_dSCFG_dTHX;
+    dTHXa(data);
 
     for (i=0; i < MP_HANDLER_NUM_PER_SRV; i++) {
         modperl_hash_handlers(aTHX_ p, s,
@@ -527,7 +521,21 @@ static int modperl_hash_handlers_srv(apr_pool_t *p, server_rec *s,
 
 void modperl_mgv_hash_handlers(apr_pool_t *p, server_rec *s)
 {
-    ap_pcw_walk_config(p, s, &perl_module, NULL,
+    MP_dINTERPa(NULL, NULL, s);
+    ap_pcw_walk_config(p, s, &perl_module,
+#ifdef USE_ITHREADS
+                       aTHX,
+#else
+                       NULL,
+#endif
                        modperl_hash_handlers_dir,
                        modperl_hash_handlers_srv);
+    MP_INTERP_PUTBACK(interp, aTHX);
 }
+
+/*
+ * Local Variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ */
